/* Copyright (C) 2023 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer] */

/// @file   CbmQaIO.cxx
/// @brief  Module for ROOT objects IO interface (implementation)
/// @author S.Zharko <s.zharko@gsi.de>
/// @since  29.03.2023

#include "CbmQaIO.h"

#include "TFile.h"

// ---------------------------------------------------------------------------------------------------------------------
//
CbmQaIO::CbmQaIO(const char* prefixName, std::shared_ptr<ObjList_t> pObjList)
  : fsPrefix(prefixName)
  , fpvObjList(pObjList)
{
  if (!fpvObjList.get()) { fpvObjList = std::make_shared<ObjList_t>(); }
}

// ---------------------------------------------------------------------------------------------------------------------
//
CbmQaIO::~CbmQaIO()
{
}

// ---------------------------------------------------------------------------------------------------------------------
//
void CbmQaIO::SetHistoProperties(TH1* pHist)
{
  pHist->SetStats(true);
  pHist->Sumw2();
}

// ---------------------------------------------------------------------------------------------------------------------
//
void CbmQaIO::WriteToFile(TFile* pOutFile) const
{
  LOG(info) << "CbmQaIO: Writing objects in file \033[1;31m" << pOutFile->GetName() << "\033[0m";
  pOutFile->cd();
  for (const auto& [pObject, sPath] : (*fpvObjList)) {
    if (!pOutFile->GetDirectory(sPath)) { pOutFile->mkdir(sPath); }
    pOutFile->cd(sPath);
    pObject->Write();
  }
}
