/* Copyright (C) 2010-2023 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Volker Friese, P.-A. Loizeau, Florian Uhlig [committer] */

std::vector<std::string> mcbm_pipevac_bmon_overlaps = {
  "cave/pipe_v19b_0/vacu20_1 overlapping cave/tof_v19e_mcbm_0/tof_v19e_mcbmStand_1/module_5_0",
  "cave/pipe_v19b_0/vacu20_1 overlapping cave/tof_v20f_mcbm_0/tof_v20f_mcbmStand_1/module_5_0",
  "cave/pipe_v19f_0/vacu20_1 overlapping cave/tof_v20c_mcbm_0/tof_v20c_mcbmStand_1/module_5_0",
  "cave/pipe_v19f_0/vacu20_1 overlapping cave/tof_v21d_mcbm_0/tof_v21d_mcbmStand_1/module_5_0",
  "cave/pipe_v19f_0/vacu20_1 overlapping cave/tof_v21f_mcbm_0/tof_v21f_mcbmStand_1/module_5_0",
  "cave/pipe_v19f_0/vacu20_1 overlapping cave/tof_v22a_mcbm_0/tof_v22a_mcbmStand_1/module_5_0",
  "cave/pipe_v19f_0/vacu20_1 overlapping cave/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_5_0"};

std::vector<std::pair<std::string, std::string>> mcbm_dets_overlaps = {
  {"gas_box extruded by: gas_box/counter_0", "between gas_box and gas_box/counter_0 (internal to mTOF v19e)"},
  {"gas_box/counter_0 overlapping gas_box/counter_1", "between counters 0 and 1 (internal to mTOF v19e)"},
  {"HalfLadder12d_Module04 extruded by: HalfLadder12d_Module04/Sensor04_1",
   ": module 4 sticking out of ladder 12 (internal to mSTS v22f)"},
  {"tof_v21j_mcbmStand/module_2_0 overlapping tof_v21j_mcbmStand/module_9_0",
   "between modules 2 and 9 (internal to mTOF v21j)"}};

std::vector<std::pair<std::string, std::string>> mcbm_dets_overlaps_sampling = {
  {"tof_v19e_mcbmStand: node module_8_0 overlapping module_8_0", "between module 8 and itself (internal to mTOF v19e)"},
  {"gas_box: node counter_0 overlapping counter_1", "between counters 0 and 1 (internal to mTOF v19e)"},
  {"gas_box: node counter_1 overlapping counter_2", "between counters 1 and 2 (internal to mTOF v19e)"},
  {"gas_box: node counter_2 overlapping counter_3", "between counters 2 and 3 (internal to mTOF v19e)"},
  {"gas_box: node counter_3 overlapping counter_4", "between counters 3 and 4 (internal to mTOF v19e)"},
  {"gas_box: node counter_4 overlapping counter_5", "between counters 4 and 5 (internal to mTOF v19e)"},
  {"gas_box: node counter_5 overlapping counter_6", "between counters 5 and 6 (internal to mTOF v19e)"},
  {"gas_box: node counter_6 overlapping counter_7", "between counters 6 and 7 (internal to mTOF v19e)"},
  {"counter: node Gap_3 overlapping tof_glass_4", "between gap and glass (internal to mTOF v21f)"},
  {"tof_v21j_mcbmStand: node module_2_0 overlapping module_9_0", "between modules 2 and 9 (internal to mTOF v21j)"}};

std::vector<std::string> cbm_pipevac_bmon_overlaps = {
  "cave/pipe_v20a_1m_0/pipevac7_0 overlapping cave/tof_v20b_1m_0/module_5_0",
  "cave/pipe_v20a_1m_0/pipe7_0 overlapping cave/tof_v20b_1m_0/module_5_0"};

std::vector<std::pair<std::string, std::string>> cbm_dets_overlaps = {
  {"cave/magnet_container_0 overlapping cave/rich_v17a_3e_0/rich_container_288", "between rich v17a_3e and magnet"},
  {"cave/magnet_container_0 overlapping cave/rich_v17a_1e_0/rich_container_290", "between rich v17a_1e and magnet"}};

std::vector<std::pair<std::string, std::string>> cbm_dets_overlaps_sampling = {
  {"cave: node magnet_container_0 overlapping rich_v17a_3e_0/rich_container_288", "between rich v17a_3e and magnet"},
  {"cave: node magnet_container_0 overlapping rich_v17a_1e_0/rich_container_290", "between rich v17a_1e and magnet"},
  {"belt_assembly_1: node belt_part3_156 overlapping belt_part4_157", "between parts of belt in ???? (s300e)"}};

bool expected_mcbm(TGeoOverlap* ov)
{
  TString OverlapName = ov->GetTitle();
  /// Bmon overlapping vacuum because introduced by mTOF which is placed in cave
  for (auto sVacBmon : mcbm_pipevac_bmon_overlaps) {
    if (OverlapName.Contains(sVacBmon)) {
      std::cout << "Expected Overlap between pipe vacuum and bmon in tof (mCBM)" << endl;
      std::cout << ov->GetTitle() << std::endl << std::endl;
      return true;
    }
  }
  /// Other detectors overlaps (either internals or caused by alignment matrices)
  for (auto pDetOver : mcbm_dets_overlaps) {
    if (OverlapName.Contains(pDetOver.first)) {
      std::cout << "Expected Overlap " << pDetOver.second << endl;
      std::cout << ov->GetTitle() << std::endl << std::endl;
      return true;
    }
  }
  return false;
}

bool expected_mcbm_sampling(TGeoOverlap* ov)
{
  TString OverlapName = ov->GetTitle();
  /// Detectors overlaps only in sampling (either internals or caused by alignment matrices)
  for (auto pDetOver : mcbm_dets_overlaps_sampling) {
    if (OverlapName.Contains(pDetOver.first)) {
      std::cout << "Expected Overlap (sampled) " << pDetOver.second << endl;
      std::cout << ov->GetTitle() << std::endl << std::endl;
      return true;
    }
  }
  return false;
}

bool expected_cbm(TGeoOverlap* ov)
{
  TString OverlapName = ov->GetTitle();
  /// Bmon overlapping vacuum because introduced by TOF which is placed in cave (ONLY IN APR21!!!)
  for (auto sVacBmon : cbm_pipevac_bmon_overlaps) {
    if (OverlapName.Contains(sVacBmon)) {
      std::cout << "Expected Overlap between pipe vacuum and bmon in tof (mCBM)" << endl;
      std::cout << ov->GetTitle() << std::endl << std::endl;
      return true;
    }
  }
  /// Detectors overlaps (either internals or caused by alignment matrices)
  for (auto pDetOver : cbm_dets_overlaps) {
    if (OverlapName.Contains(pDetOver.first)) {
      std::cout << "Expected Overlap " << pDetOver.second << endl;
      std::cout << ov->GetTitle() << std::endl << std::endl;
      return true;
    }
  }
  return false;
}

bool expected_cbm_sampling(TGeoOverlap* ov)
{
  TString OverlapName = ov->GetTitle();
  /// Detectors overlaps only in sampling (either internals or caused by alignment matrices)
  for (auto pDetOver : cbm_dets_overlaps_sampling) {
    if (OverlapName.Contains(pDetOver.first)) {
      std::cout << "Expected Overlap (sampled) " << pDetOver.second << endl;
      std::cout << ov->GetTitle() << std::endl << std::endl;
      return true;
    }
  }
  return false;
}

void check_overlaps(TString dataset = "test")
{
  // 2014-07-04 - DE - test CBM setups for collisions in nightly tests
  // 2014-07-04 - DE - currently there are 2 overlaps between the PIPE and STS layer 8
  // 2014-07-04 - DE - set the default to 0 overlaps, anyway
  // 2017-29-11 - FU - define some expected overlaps between magnet and rich or much
  //                   these overlas are accepted for the time being until
  //                   there is a new magnet geometry

  UInt_t unexpectedOverlaps {0};

  TString geoFile = dataset + ".geo.root";
  TFile* f        = new TFile(geoFile);
  if (!f->IsOpen()) {
    std::cout << "check_overlaps: geometry file " << geoFile << " is not accessible!" << std::endl;
    return;
  }

  gGeoManager = (TGeoManager*) f->Get("FAIRGeom");

  gGeoManager->CheckOverlaps(0.0001);
  TIter next(gGeoManager->GetListOfOverlaps());
  TGeoOverlap* ov;
  while ((ov = (TGeoOverlap*) next())) {
    TString OverlapName = ov->GetTitle();

    if ((dataset.Contains("mcbm") && expected_mcbm(ov)) || expected_cbm(ov)) {
      /// Detected and logged in special function above
      continue;
    }
    else {
      cout << "Unexpected Overlap:" << endl;
      ov->PrintInfo();
      cout << endl;
      unexpectedOverlaps++;
    }
  }
  std::cout << std::endl;

  gGeoManager->CheckOverlaps(0.0001, "s");
  TIter next1(gGeoManager->GetListOfOverlaps());
  while ((ov = (TGeoOverlap*) next1())) {

    if ((dataset.Contains("mcbm") && expected_mcbm_sampling(ov)) || expected_cbm_sampling(ov)) {
      /// Detected and logged in special function above
      continue;
    }
    else {
      std::cout << "Unexpected Overlap (sampled):" << std::endl;
      ov->PrintInfo();
      std::cout << std::endl;
      unexpectedOverlaps++;
    }
  }
  std::cout << std::endl;

  if (unexpectedOverlaps != 0) {
    std::cout << " Test failed" << std::endl;
    std::cout << " We have in total " << unexpectedOverlaps << " unexpected overlaps." << std::endl;
  }
  else {
    std::cout << " There are no unexpected overlaps." << std::endl;
    std::cout << " Test passed" << std::endl;
    std::cout << " All ok " << std::endl;
  }


  //RemoveGeoManager();
}
